from django.test import TestCase, Client
from django.contrib.auth.models import *
from django.core.cache import cache
from django.core.urlresolvers import reverse
from django.core.exceptions import PermissionDenied
from django.conf import settings
from django.db.models.signals import post_save
from key.models import *
from key.signals import *
import logging
import hashlib
import test_urls

class ApiKeyTest(TestCase):
    def __init__(self, *args, **kwargs):
        settings.USE_API_GROUP = kwargs.get('USE_API_GROUP', False)
        if kwargs.has_key('USE_API_GROUP'):
            del kwargs['USE_API_GROUP']
        super(ApiKeyTest, self).__init__(*args, **kwargs)

    def setUp(self):
        cache.clear()
        create_group()
        self.user = User.objects.create_user(username='ApiKeyTest',
                                             email='ApiKeyTest@example.com',
                                             password='ApiKeyTestPassword')
        if USE_API_GROUP:
            gr = Group.objects.get(name="API User")
            self.user.groups.add(gr)
            self.user.save()
            gr.save()
        else:
            assign_api_key_permissions(self.user)
        k = generate_unique_api_key(self.user)
        self.assertTrue(k is not None)
        self.assertNotEquals(k.key, None)
        kstr = k.key
        self.unauthorized_user = User.objects.create_user(username="NonAuthorized",
                                                          email="NonAuthorized@example.com",
                                                          password="NonAuthorizedPassword")
        self.unauthorized_user.save()

    def test_key_profile(self):
        profile = self.user.key_profile
        self.assertNotEquals(profile, None)
        self.assertEquals(profile.user, self.user)
        self.assertEquals(profile.max_keys, MAX_KEYS)
        self.profile_signaled = False
        def check_signal(sender, instance, created, *args, **kwargs):
            self.profile_signaled = True
        post_save.connect(check_signal, sender=ApiKeyProfile,
                          dispatch_uid='test_check_signal')
        k = profile.api_keys.all()[0]
        k.save()
        self.assertTrue(self.profile_signaled)

    def test_key_generation(self):
        k = generate_unique_api_key(self.user)
        ksize = KEY_SIZE
        my_kstr = hashlib.md5('%s-%s' % (self.user.email,
                                         k.created)).hexdigest()[:ksize]
        self.assertTrue(self.user.key_profile.can_make_api_key())
        self.assertEquals(self.user.key_profile.available_keys(), 'Unlimited')
        self.assertEquals(len(my_kstr), len(k.key))
        self.assertEquals(my_kstr, k.key)
        self.assertEquals(self.user.key_profile.api_keys.count(), 2)
        k.delete()
        self.assertEquals(self.user.key_profile.api_keys.count(), 1)        
        current_max = self.user.key_profile.max_keys
        self.user.key_profile.max_keys = 2
        self.user.key_profile.save()
        self.assertEquals(self.user.key_profile.max_keys, 2)
        self.assertEquals(self.user.key_profile.available_keys(), 1)
        self.assertTrue(self.user.key_profile.can_make_api_key())
        self.user.key_profile.max_keys = 1
        self.user.key_profile.save()
        self.assertEquals(self.user.key_profile.max_keys, 1)
        self.assertEquals(self.user.key_profile.available_keys(), 0)
        self.assertFalse(self.user.key_profile.can_make_api_key())
        self.user.key_profile.max_keys = current_max
        self.user.key_profile.save()
        k = self.user.key_profile.api_keys.all()[0]
        self.assertEquals(self.user.key_profile.__unicode__(), 'ApiKeyProfile: 1, %s' % current_max)
        self.assertEquals(k.__unicode__(), 'ApiKey: %s' % k.key)

    def test_authentication(self):
        def do_test_authentication(auth_header):
            client = Client()
            def t_user_logged_in(*args, **kwargs):
                self.user_signalled = True
            auth_header = 'HTTP_%s' % (auth_header.upper().replace('-', '_'))
            key = self.user.key_profile.api_keys.all()[0]
            extra = {auth_header: key.key}
            self.user_signalled = False
            user_logged_in.connect(t_user_logged_in, dispatch_uid='t_user_logged_in')
            
            rv = client.get(reverse('test_key_list'), **extra)
            self.assertEquals(rv.status_code, 200)
            key.login('127.0.0.1')
            self.assertEquals(key.logged_ip, '127.0.0.1')
            self.assertTrue(self.user_signalled)
            
            rv = client.get(reverse('test_key_view', args=(key.pk,)))
            self.assertEquals(rv.status_code,401)
            rv = client.get(reverse('test_key_view', args=(key.pk,)), **extra)
            self.assertEquals(rv.status_code,200)
            rv = client.get(reverse('test_key_view', args=(key.pk+1,)), **extra)
            self.assertEquals(rv.status_code, 404)
            
            rv = client.get(reverse('test_key_list'))
            self.assertEquals(rv.status_code, 401)
            self.user.key_profile.api_keys.all()[0].logout()
            key.logout()
            self.assertEquals(key.logged_ip, None)
        import key.settings
        original = getattr(settings, 'APIKEY_AUTHORIZATION_HEADER', 'X-Api-Authorization')
        setattr(settings, 'APIKEY_AUTHORIZATION_HEADER', 'X-Api-Authorization')
        key.settings.reload()
        self.assertEquals(key.settings.AUTH_HEADER, 'X-Api-Authorization')
        do_test_authentication('X-Api-Authorization')
        setattr(settings, 'APIKEY_AUTHORIZATION_HEADER', 'X-MyCoolApp-Key')
        key.settings.reload()
        self.assertEquals(key.settings.AUTH_HEADER, 'X-MyCoolApp-Key')
        do_test_authentication('X-MyCoolApp-Key')
        setattr(settings, 'APIKEY_AUTHORIZATION_HEADER', original)
        key.settings.reload()
        self.assertEquals(key.settings.AUTH_HEADER, original)
        
    def test_perm_check(self):
        client = Client()
        client.login(username="NonAuthorized", password="NonAuthorizedPassword")
        rv = client.get(reverse('key.create'))
        self.assertEquals(rv.status_code, 302)
        self.assertRaises(PermissionDenied,
                          generate_unique_api_key,
                          self.unauthorized_user)
        
    def test_views(self):
        client = Client()
        rv = client.get(reverse('key.list'))
        self.assertEquals(rv.status_code, 302)
        rv = client.get(reverse('key.create'))
        self.assertEquals(rv.status_code, 302)
        rv = client.get(reverse('key.delete',args=('ValueDoesntMatter',)))
        self.assertEquals(rv.status_code, 302)
        client = Client()
        self.assertTrue(client.login(username='ApiKeyTest',
                                     password='ApiKeyTestPassword'))
        u = User.objects.get(username='ApiKeyTest')
        self.assertTrue(u.check_password('ApiKeyTestPassword'))
        self.assertTrue(u.has_perm('key.can_make_api_key'))
        self.assertTrue(u.has_perm('key.has_api_key_profile'))
        rv = client.get(reverse('key.list'))
        self.assertEquals(rv.status_code, 200)
        original_list = list(self.user.key_profile.api_keys.all())
        self.assertEquals(self.user.key_profile.api_keys.count(), 1)
        rv = client.post(reverse('key.create'))
        self.assertEquals(rv.status_code, 302)
        self.assertEquals(rv['Location'], 
                          'http://testserver' + reverse('key.list'))
        rv = client.get(rv['Location'])
        self.assertEquals(rv.status_code, 200)
        self.assertEquals(self.user.key_profile.api_keys.count(), 2)        
        k = self.user.key_profile.api_keys.latest('created')
        self.assertEquals(k, k)
        [self.assertNotEquals(k, x) for x in original_list]
        found = False
        self.assertTrue(k in self.user.key_profile.api_keys.all())
        rv = client.post(reverse('key.delete',args=(k.key,)), {'action': 'Cancel'})
        self.assertEquals(rv.status_code, 302)
        rv = client.post(reverse('key.delete',args=(k.key,)))
        self.assertTrue(k not in self.user.key_profile.api_keys.all())
        self.assertEquals(rv.status_code, 302)
        self.assertEquals(rv['Location'],
                          'http://testserver' + reverse('key.list'))
        self.assertEquals(self.user.key_profile.api_keys.count(), 1)
        rv = client.get('/admin/key/')
        self.assertEquals(rv.status_code, 200)
        rv = client.get('/admin/key/apikey/1/')
        self.assertEquals(rv.status_code, 200)
        rv = client.get('/admin/key/apikeyprofile/1/')
        self.assertEquals(rv.status_code, 200)
        self.user.key_profile.api_keys.all().delete()

class ApiKeyGroupTest(ApiKeyTest):
    def __init__(self, *args, **kwargs):
        super(ApiKeyGroupTest, self).__init__(USE_API_GROUP=True, *args, **kwargs)



from django.conf.urls.defaults import *
from django.views.decorators.cache import cache_page
from piston.resource import Resource
from piston.handler import BaseHandler, AnonymousBaseHandler
from piston.utils import *
from key.auth import *
import key.urls

class KeyHandler(BaseHandler):
    allowed_methods = ('GET',)
    model = ApiKey
    
    def read(self, request):
        return ApiKey.objects.filter(profile=request.user.key_profile)
        

class KeyViewHandler(BaseHandler):
    allowed_methods = ('GET',)
    model = ApiKey

    def read(self, request, key_pk):
        try:
            return ApiKey.objects.get(profile=request.user.key_profile,
                                     pk=key_pk)
        except self.model.DoesNotExist:
            return rc.NOT_FOUND

auth = ApiKeyAuthentication()
key_handler = Resource(handler=KeyHandler,
                       authentication=auth)
key_view_handler = Resource(handler=KeyViewHandler,
                            authentication=auth)

test_patterns = patterns('',                       
                         url(r'^k/$', key_handler, name="test_key_list" ),
                         url(r'^k/(?P<key_pk>\d+?)$', key_view_handler, name="test_key_view"),
                         )

key.urls.urlpatterns = key.urls.urlpatterns + test_patterns



from django.conf import settings
from django.http import HttpResponse, Http404, HttpResponseRedirect
from django.template import RequestContext
from django.shortcuts import render_to_response, get_object_or_404, redirect
from django.contrib.auth.decorators import login_required, permission_required
from django.views.generic import *
from django.views.generic.base import TemplateResponseMixin, View
from django.utils.decorators import method_decorator
from django.views.decorators.http import condition, last_modified
from django.views.decorators.cache import cache_control, cache_page
from django.core.urlresolvers import reverse
from django.core.cache import cache
from key.models import ApiKey, generate_unique_api_key
from key.forms import *
from django.contrib.auth.models import User
import hashlib
import datetime
import logging

class ProtectedView(object):
    @method_decorator(permission_required('key.has_api_key_profile'))
    def dispatch(self, *args, **kwargs):
        return super(ProtectedView, self).dispatch(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(ProtectedView, self).get_context_data(**kwargs)
        context['request'] = self.request
        return context

class KeyCreateView(ProtectedView, CreateView):
    def get_queryset(self):
        profile = self.request.user.key_profile
        return ApiKey.objects.filter(profile=profile)
    
    def get_success_url(self):
        return reverse('key.list')

    def post(self, request, *args, **kwargs):
        self.object = generate_unique_api_key(self.request.user)
        self.object.save()
        return HttpResponseRedirect(self.get_success_url())
    
        
class KeyListView(ProtectedView, ListView):
    def get_queryset(self):
        profile = self.request.user.key_profile
        return ApiKey.objects.filter(profile=profile)

    def get_context_data(self, **kwargs):
        context = super(KeyListView, self).get_context_data(**kwargs)
        user = self.request.user
        context['user'] = user
        context['can_make_api_key'] = user.key_profile.can_make_api_key()
        context['available_keys'] = user.key_profile.available_keys()
        return context

class KeyDeleteView(ProtectedView, DeleteView):
    def get_success_url(self):
        return reverse('key.list')

    def get_object(self):
        return self.get_queryset().get(key=self.kwargs['key_code'])

    def get_queryset(self):
        profile = self.request.user.key_profile
        return profile.api_keys.all()

    def post(self, request, *args, **kwargs):
        if self.request.POST.get('action') == 'Cancel':
            return HttpResponseRedirect(self.get_success_url())
        return super(KeyDeleteView, self).post(request, *args, **kwargs)




from django.contrib.auth.models import User, Permission, user_logged_in
from django.core.exceptions import PermissionDenied
from datetime import datetime
from django.db.models.signals import post_save
from django.db.models.signals import post_delete
from key.models import *
from key.signals import *
import logging
import time
import hashlib

def perm_check(user):
    try:
        if user.key_profile:
            return
    except:
        if user.has_perm("key.has_api_key_profile"):
            profile, created = ApiKeyProfile.objects.get_or_create(user=user)
            if created:
                api_user_created.send(sender=user.__class__,
                                      instance=user)
            else:
                raise PermissionDenied
        else:
            raise PermissionDenied

def generate_unique_key_code(user):
    perm_check(user)
    while True:
        now = datetime.utcnow()
        kstr = hashlib.md5('%s-%s' % (user.email, now)).hexdigest()[:KEY_SIZE]
        key, created = ApiKey.objects.get_or_create(profile=user.key_profile,
                                                    key=kstr,
                                                    created=now)
        if created:
            try:
                key.save()
                return key
            except IntegriyError:
                pass
        time.sleep(0.01)

def generate_unique_api_key(user):
    key = generate_unique_key_code(user)
    api_key_created.send(sender=key.__class__, instance=key)
    return key

def update_profile_timestamps(sender, instance, created, *args, **kwargs):
    instance.profile.last_access = datetime.utcnow()
    instance.profile.save()
post_save.connect(update_profile_timestamps, sender=ApiKey, dispatch_uid='update_profile_timstamps')

def create_key_profile(user):
    profile, c = ApiKeyProfile.objects.get_or_create(user=user)
    return profile




from django.conf import settings

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'my_db', 
        }
    }
INSTALLED_APPS = ['key', 'django.contrib.auth', 'django.contrib.contenttypes', 'django.contrib.sessions', 'django.contrib.admin']


ROOT_URLCONF = 'key.urls'

MAX_KEYS = getattr(settings, 'APIKEY_MAX_KEYS', -1)
KEY_SIZE = getattr(settings, 'APIKEY_KEY_SIZE', 32)
USE_API_GROUP = getattr(settings, 'APIKEY_USE_API_GROUP', False)
AUTH_HEADER =  getattr(settings, 'APIKEY_AUTHORIZATION_HEADER', 'X-Api-Authorization')


def reload():
    global MAX_KEYS, KEY_SIZE, USE_API_GROUP, AUTH_HEADER
    MAX_KEYS = getattr(settings, 'APIKEY_MAX_KEYS', -1)
    KEY_SIZE = getattr(settings, 'APIKEY_KEY_SIZE', 32)
    USE_API_GROUP = getattr(settings, 'APIKEY_USE_API_GROUP', False)
    AUTH_HEADER =  getattr(settings, 'APIKEY_AUTHORIZATION_HEADER', 'X-Api-Authorization')




# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Deleting model 'ApiKey'
        db.delete_table('key_apikey')

        # Deleting model 'ApiKeyProfile'
        db.delete_table('key_apikeyprofile')


    def backwards(self, orm):
        
        # Adding model 'ApiKey'
        db.create_table('key_apikey', (
            ('profile', self.gf('django.db.models.fields.related.ForeignKey')(related_name='api_keys', to=orm['key.ApiKeyProfile'])),
            ('last_used', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.utcnow)),
            ('logged_ip', self.gf('django.db.models.fields.CharField')(default=None, max_length=32, null=True, blank=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.utcnow)),
            ('key', self.gf('django.db.models.fields.CharField')(default=None, max_length=16, unique=True, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
        ))
        db.send_create_signal('key', ['ApiKey'])

        # Adding model 'ApiKeyProfile'
        db.create_table('key_apikeyprofile', (
            ('max_keys', self.gf('django.db.models.fields.IntegerField')(default=-1)),
            ('last_access', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.utcnow)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.OneToOneField')(related_name='key_profile', unique=True, to=orm['auth.User'])),
        ))
        db.send_create_signal('key', ['ApiKeyProfile'])


    models = {
        
    }

    complete_apps = ['key']



from django.http import HttpResponse
from django.core import cache
from django.contrib.auth.models import *
from key.models import ApiKey
from key import settings
import logging
import hashlib

def _key_cache_key(auth_string):
    kstr = 'key.%s' % auth_string
    return hashlib.md5(kstr).hexdigest()

class ApiKeyAuthentication(object):
    def is_authenticated(self, request):
        auth_header = getattr(settings, 'AUTH_HEADER')
        auth_header = 'HTTP_%s' % (auth_header.upper().replace('-', '_'))
        auth_string = request.META.get(auth_header)
        if not auth_string:
            return False
        try:
            user = cache.get_cache('default').get(_key_cache_key(auth_string))
            if user:
                request.user = user
            else:
                key = ApiKey.objects.get(key=auth_string)
                key.login(request.META.get('REMOTE_ADDR'))
                request.user = key.profile.user
                cache.get_cache('default').set(_key_cache_key(key.key), request.user)
                if not key.profile.user.has_perm('key.can_use_api'):
                    return False                
                user_logged_in.send(sender=request.user.__class__,
                                    request=request, user=request.user)
                
        except ApiKey.DoesNotExist:
            return False
        request.key = auth_string
        return True
        
    def challenge(self):
        auth_header = getattr(settings, 'AUTH_HEADER')
        resp = HttpResponse('Authorization Required')
        resp['WWW-Authenticate'] = 'KeyBasedAuthentication realm="API"'       
        resp[auth_header] = 'Key Needed'
        resp.status_code = 401
        return resp



from key.models.models import *
from key.models.actions import *

__all__ = ['ApiKeyProfile', 
           'ApiKey', 
           'generate_unique_key_code',
           'generate_unique_api_key',
           'create_group',
           'assign_api_key_permissions',
           'MAX_KEYS',
           'USE_API_GROUP',
           'KEY_SIZE',
           ]




# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Changing field 'ApiKey.key'
        db.alter_column('key_apikey', 'key', self.gf('django.db.models.fields.CharField')(unique=True, max_length=16))


    def backwards(self, orm):
        
        # Changing field 'ApiKey.key'
        db.alter_column('key_apikey', 'key', self.gf('django.db.models.fields.CharField')(max_length=32, unique=True))


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'key.apikey': {
            'Meta': {'ordering': "['-created']", 'object_name': 'ApiKey'},
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.utcnow'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'key': ('django.db.models.fields.CharField', [], {'default': 'None', 'unique': 'True', 'max_length': '16', 'blank': 'True'}),
            'last_used': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.utcnow'}),
            'logged_ip': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '32', 'null': 'True', 'blank': 'True'}),
            'profile': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'api_keys'", 'to': "orm['key.ApiKeyProfile']"})
        },
        'key.apikeyprofile': {
            'Meta': {'object_name': 'ApiKeyProfile'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_access': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.utcnow'}),
            'max_keys': ('django.db.models.fields.IntegerField', [], {'default': '-1'}),
            'user': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'key_profile'", 'unique': 'True', 'to': "orm['auth.User']"})
        }
    }

    complete_apps = ['key']



from django.conf.urls.defaults import *
from key.views import *

urlpatterns = patterns('key.views',
                       url(r'^create_key/$', KeyCreateView.as_view(),
                           name='key.create' ),
                       url(r'^keys/$', KeyListView.as_view(),
                           name='key.list' ),
                       url(r'^delete_key/(?P<key_code>.*)/$', 
                           KeyDeleteView.as_view(),
                           name='key.delete' ),
                       )

import sys
if 'test' in sys.argv:
    from django.contrib import admin
    admin.autodiscover()
    urlpatterns += patterns('', url(r'^admin/', include(admin.site.urls)))



from django.dispatch import Signal

api_user_created = Signal(providing_args=['instance'])
api_key_created = Signal(providing_args=['instance'])
api_user_logged_in = Signal(providing_args=['instance'])



from django.contrib import admin
from key.models import *
from key.admin.forms import *

class ApiKeyInline(admin.TabularInline):
    model = ApiKey
    extra = 0
    max_num = 0
    fields = ('key', 'logged_ip', 'last_used', 'created')
    readonly_fields = ('key', 'logged_ip')

class ApiKeyAdmin(admin.ModelAdmin):
    form = ApiKeyAdminForm

    def has_add_permission(self, *args, **kwargs):
        return False

    def queryset(self, request):
        if request.user.is_superuser:
            return ApiKey.objects.all()
        else:
            p = ApiKeyProfile.objects.get(user=request.user)
            return p.api_keys.all()

class ApiKeyProfileAdmin(admin.ModelAdmin):
    form = ApiKeyProfileAdminForm
    inlines = (ApiKeyInline,)

    def has_add_permission(self, *args, **kwargs):
        return False


admin.site.register(ApiKey, ApiKeyAdmin)
admin.site.register(ApiKeyProfile, ApiKeyProfileAdmin)





from django.db import models
from django.contrib.auth.models import User, Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.conf import settings
from datetime import datetime
from django.db.utils import IntegrityError
import logging
import time
import hashlib
from key.settings import MAX_KEYS, KEY_SIZE, USE_API_GROUP

class ApiKeyProfile(models.Model):
    user = models.OneToOneField(User, related_name='key_profile')
    max_keys = models.IntegerField(default=MAX_KEYS)
    last_access = models.DateTimeField(default=datetime.utcnow)

    class Meta:
        app_label = 'key'

    def available_keys(self):
        if self.max_keys == -1:
            return 'Unlimited'
        return self.max_keys - self.api_keys.count()

    def can_make_api_key(self):
        if self.available_keys() > 0:
            return True

    def __unicode__(self):
        return "ApiKeyProfile: %s, %s" % (self.api_keys.count(),
                                          self.max_keys)

class ApiKey(models.Model):
    profile = models.ForeignKey(ApiKeyProfile, related_name='api_keys')
    key = models.CharField(max_length=KEY_SIZE, unique=True, blank=True, default='')
    logged_ip = models.CharField(max_length=32, blank=True, null=True, default=None)
    last_used = models.DateTimeField(default=datetime.utcnow)
    created = models.DateTimeField(default=datetime.utcnow)
    
    class Meta:
        app_label = 'key'
        ordering = ['-created']

    def login(self, ip_address):
        self.logged_ip = ip_address
        self.save()

    def logout(self):
        self.logged_ip = None
        self.save()

    def __unicode__(self):
        return 'ApiKey: %s' % (self.key)

def assign_api_key_permissions(user_or_group):
    ct = ContentType.objects.get(app_label="key", model="apikey")
    can_gen, pc = Permission.objects.get_or_create(name="Can generate an API key",
                                             codename="can_make_api_key",
                                             content_type=ct)
    ct = ContentType.objects.get(app_label='key', model='apikeyprofile')
    has_prof, pc = Permission.objects.get_or_create(name="Has an API key profile",
                                             codename="has_api_key_profile",
                                             content_type=ct)
    can_login, pc = Permission.objects.get_or_create(name="Can use the API",
                                                     codename="can_use_api",
                                                     content_type=ct)
    perm_list = getattr(user_or_group, 'permissions', 
                        getattr(user_or_group, 'user_permissions'))
    perm_list.add(can_gen)
    perm_list.add(has_prof)
    perm_list.add(can_login)
    for permission in [ 'add_apikey', 'change_apikey', 'delete_apikey' ]:
        p = Permission.objects.get(codename=permission)
        perm_list.add(p)
    p = Permission.objects.get(codename='change_apikeyprofile')
    perm_list.add(p)
    user_or_group.save()
    return user_or_group
    
def create_group():
    if USE_API_GROUP:
        gr, cr = Group.objects.get_or_create(name='API User')
        if cr:
            assign_api_key_permissions(gr)
        return gr



from django.forms import *
import logging
from key.models import *

class ApiKeyProfileAdminForm(ModelForm):
    class Meta:
        model = ApiKeyProfile
        readonly_fields = ('last_access',)

    def clean(self, *args, **kwargs):
        cleaned_data = super(ApiKeyProfileAdminForm, self).clean(*args, **kwargs)
        if self.data.has_key('generate'):
            generate_unique_api_key(self.instance.user)
        return cleaned_data

class ApiKeyAdminForm(ModelForm):
    class Meta:
        model = ApiKey
        readonly_fields = ('key', 'logged_ip')

    def save(self, commit=True, *args, **kwargs):
        model = super(ApiKeyAdminForm, self).save(commit=False, *args, **kwargs)
        if not model.key:
            model = generate_unique_key_code(model.profile.user, model)
        if commit:
            model.save()
        return model
